// String-vector editor: use for lists of files (variable number of them)
// This is a special TR4-string version, because TR4 can have long strings

package tr4_textedit;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import util.*;


final class StringVecEditor extends Dialog {
	
	// Stuff that has to be local
	private Frame Parent;
	private Vector StringVec;
	
	// Displayed list of strings:
	private List DispList;
	
	// Field for editing string:
	private TextArea EditField;
	
	// Create a string in a suitable format for the display list:
	private String MakeDispString(int ic, String ListStr) {
		return (Integer.toString(ic) + ": " + ListStr);
	}
	
	private static final int TruncateLen = 32;
	
	// Truncate a string for displaying in the list
	private String Truncate(String S) {
		int SLen = S.length();
		
		// Truncate if necessary and indicate the truncation
		if (SLen > TruncateLen)
			return (S.substring(0,TruncateLen-1) + " ...");
		
		// Unchanged...
		return S;
	}
	
	// Rebuild the display list
	private void MakeDispList() {
		DispList.removeAll();
		for (int ic=0; ic<StringVec.size(); ic++)
			DispList.add(MakeDispString(ic,Truncate((String)StringVec.elementAt(ic))));
	}
		
	StringVecEditor(Frame _Parent, Vector _StringVec, String Name) {
		super(_Parent,Name);
		setSize(384,256);
		setResizable(false);
		
		// Necessary for some of the inner classes
		Parent = _Parent;
		StringVec = _StringVec;
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
		
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// The display list itself
		DispList = new List(20);
		MakeDispList();
		
		// Action is to get a string from the list
		DispList.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				EditField.setText((String)StringVec.elementAt(ic));
			}}
		);
		
		C.gridx = 0;
		C.gridheight = 16;
		C.anchor = GridBagConstraints.NORTHWEST;
		add(DispList,C);
		C.gridheight = 1;
		
		// The edit-text field
		EditField = new TextArea(6,40);
		int Row = 0;
		C.gridx = 1;
		C.gridy = (Row++);
		C.anchor = GridBagConstraints.WEST;
		add(EditField,C);
		
		// The control buttons
		C.fill = GridBagConstraints.HORIZONTAL;
		C.anchor = GridBagConstraints.WEST;
		
		// Get the currently-selected string
		Button GetButton = new Button("Get from List");
		GetButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				EditField.setText((String)StringVec.elementAt(ic));
			}}
		);
		C.gridy = (Row++);
		add(GetButton,C);
		
		// Set the currently-selected string to the text-field contents
		Button ChangeButton = new Button("Change in List");
		ChangeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				String NewString =  EditField.getText();
				StringVec.setElementAt(NewString,ic);
				DispList.replaceItem(MakeDispString(ic,NewString),ic);
					
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(ChangeButton,C);

		// Duplicate the current string
		Button DuplicateButton = new Button("Duplicate");
		DuplicateButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				String Str = new String((String)StringVec.elementAt(ic));
				StringVec.insertElementAt(Str,ic);
				MakeDispList();
				DispList.select(ic);
			}}
		);
		C.gridy = (Row++);
		add(DuplicateButton,C);

		// Move the current string to some location
		Button MoveButton = new Button("Move");
		MoveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				int SVLen = StringVec.size();
				String[] LocList = new String[SVLen + 1];
				for (int ix=0; ix<SVLen; ix++) {
					LocList[ix] = Truncate((String)StringVec.elementAt(ix));
				}
				LocList[SVLen] = "<<< End >>>";
				String Str = (String)StringVec.elementAt(ic);
				StringLocSelector SLE = 
					new StringLocSelector(Parent,"String-List Editor",
					"Move string \""+Str+"\" to before",LocList);
				if (SLE.DoGoto) {
					// This bumps down all strings after it
					StringVec.removeElementAt(ic);
					int Loc = SLE.Loc;
					if (Loc > ic) Loc--;
					StringVec.insertElementAt(Str,Loc);
					MakeDispList();
					DispList.select(Loc);
				}
			}}
		);
		C.gridy = (Row++);
		add(MoveButton,C);

		// Insert a string before the currently-selected string location
		Button InsertBeforeButton = new Button("Insert Before");
		InsertBeforeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (StringVec.size() <= 0) {
					StringVec.addElement(EditField.getText());
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					StringVec.insertElementAt(EditField.getText(),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertBeforeButton,C);
		
		// Insert a string after the currently-selected string location
		Button InsertAfterButton = new Button("Insert After");
		InsertAfterButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (StringVec.size() <= 0) {
					StringVec.addElement(EditField.getText());
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					ic++;
					StringVec.insertElementAt(EditField.getText(),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertAfterButton,C);

		// Delete the currently-selected string
		Button DeleteButton = new Button("Delete");
		DeleteButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				WarningAlert A = new WarningAlert(Parent,"Warning!",384,72,
					"Really want to delete \"" + (String)StringVec.elementAt(ic) + "\"?");
				if (A.If_OK) {
					StringVec.removeElementAt(ic);
					MakeDispList();
					DispList.select(ic);
				
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(DeleteButton,C);
		
		// Load the strings from a file
		Button LoadButton = new Button("Load Strings");
		LoadButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Load list from file",FileDialog.LOAD);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File InFile = new File(DirName,FileName);
				
				FileReader FR;
				
				StringVec.removeAllElements();
				
				try {
					FR = new FileReader(InFile);
					
					// Java's StreamTokenizer does not seem to recognize escape characters :-P
					// so I have to read in byte by byte
					// Escaping tells whether the escape character, '\', had been reached.
					// Attempt to parse MacOS, DOS, and Unix ends of lines correctly.
					boolean Escaping = false;
					// Java does not seem to have enums...
					final int EOL_None = 0;
					final int EOL_CR = 1;
					final int EOL_LF = 2;
					int EOL_State = EOL_None;
					StringBuffer ReadString = null;
					while(true) {
						int ic = FR.read();
						if (ic == -1) break;
						char c = (char)ic;
						
						if (Escaping) {
							if (c == '\r') {
								EOL_State = EOL_CR;
								c = '\n';
							} else if (c == '\n') {
								EOL_State = EOL_LF;
							}
							Escaping = false;
						} else if (c == '\\') {
							Escaping = true;
							EOL_State = EOL_None;
							continue;
						} else if (c == '\r') {
							// Always the end of a line:
							EOL_State = EOL_CR;
							
							// Add a string if a non-escaped end of line
							if (ReadString != null)
								StringVec.addElement(ReadString.toString());
							ReadString = null;
							continue;
						} else if (c == '\n') {
							// Skip over if there was a CR before it (DOS case)
							if (EOL_State == EOL_CR) {
								EOL_State = EOL_None;
								continue;
							}
							EOL_State = EOL_LF;
							
							// Add a string if a non-escaped end of line
							if (ReadString != null)
								StringVec.addElement(ReadString.toString());
							ReadString = null;
							continue;
						} else {
							EOL_State = EOL_None;
						}
						
						// Add the character; create a string for it if necessary
						if (ReadString == null)
							ReadString = new StringBuffer();
						ReadString.append(c);
					}
					if (ReadString != null)
						StringVec.addElement(ReadString.toString());
					FR.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Load Error for " + FileName, 480,48, X.toString());
				} finally {
					MakeDispList();
					
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(LoadButton,C);
		
		// Save the strings to a file
		Button SaveButton = new Button("Save Strings");
		SaveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Save list in file",FileDialog.SAVE);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File OutFile = new File(DirName,FileName);
				
				FileWriter FW;
				
				try {
					FW = new FileWriter(OutFile);
				
					for (int ic=0; ic<StringVec.size(); ic++) {
						String S = (String)StringVec.elementAt(ic);
						// Write character by character, looking for backslashes and newlines,
						// which get treated separately.
						for (int i=0; i<S.length(); i++) {
							char c = S.charAt(i);
							if (c == '\\')
								// Double a backslash to indicate its presence
								FW.write("\\\\");
							else if (c == '\n') {
								FW.write("\\");
								FW.write(System.getProperty("line.separator"));
							} else
								FW.write(c);
						}
						FW.write(System.getProperty("line.separator"));
					}
					FW.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Save Error for " + FileName, 480,48, X.toString());
					return;
				}
			}}
		);
		C.gridy = (Row++);
		add(SaveButton,C);
				
		// Close this dialog box (changes are saved automatically)
		Button CloseButton = new Button("Close");
		CloseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {dispose();}}
		);
		C.gridy = (Row++);
		add(CloseButton,C);

		// Handle the close box:
		addWindowListener(new WindowListener()
			{
			public void windowOpened(WindowEvent E) {}
			public void windowClosing(WindowEvent E) {dispose();}
			public void windowClosed(WindowEvent E) {}
			public void windowIconified(WindowEvent E) {}
			public void windowDeiconified(WindowEvent E) {}
			public void windowActivated(WindowEvent E) {}
			public void windowDeactivated(WindowEvent E) {}
			}
		);
		
		validate();
		setVisible(true);
		
		setSize(getPreferredSize());
		validate();
	}
}

