/*

	This is the script-editor master object, which contains the script data and
	which contains the master dialog box for editing
	
*/

package tr4_textedit;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import util.*;


// It's a subclass of Dialog since a dialog box is an intimate part of the editor
final class TextSetEditor extends Dialog {

	// Text-string data to be edited
	TextSet TS;
	
	// Parent frame
	private Frame Parent;
	
	// Make the save directory and filename persistent
	private String DirName, FileName;
	
	// Close-window action:
	private void CloseAction() {
		WarningAlert A = new WarningAlert(Parent,"Warning!",384,72,
			"Really want to stop editing " + TS.FileName + "?");
		if (A.If_OK) dispose();
	}
	
	// Needs a parent frame
	TextSetEditor(Frame _Parent, TextSet _TS) {
		super(_Parent,_TS.FileName + " Editor");
		setResizable(false);
		setSize(192,192);
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
		
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// Plug in
		Parent = _Parent;
		TS = _TS;
		
		int Row = 0;
		
		// Start new column for the various control buttons
		C.fill = GridBagConstraints.HORIZONTAL;
		Row = 1;
		
		// Edit the various kinds of strings
		for (int i=0; i<TextSet.NumTextBlocks; i++) {
			final TextStringBlock TSB = TS.TextBlockSet[i];
					
			Button EditTSButton = new Button("Edit " + TSB.TypeName);
			EditTSButton.addActionListener(new ActionListener()
				{public void actionPerformed(ActionEvent E) {
					StringVecEditor SVE =
						new StringVecEditor(Parent,TSB.Strings,TS.FileName + " " + TSB.TypeName);
					}}
			);
			add(EditTSButton,C);
			C.gridy = (Row++);
		}
		
		// Save this file
		Button SaveButton = new Button("Save");
		SaveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				
				FileDialog FD = new FileDialog(Parent,"Save <language>.DAT file",FileDialog.SAVE);
				FD.setDirectory(DirName);
				FD.setFile(FileName);
				FD.setVisible(true);
				DirName = FD.getDirectory();
				FileName = FD.getFile();
				if (FileName == null) return;
				File OutFile = new File(DirName,FileName);
		
				OutputStream OS;
				
				try {
					OS = (OutputStream)new FileOutputStream(OutFile);
					TextSetIO.Write(Parent, OS, TS);
					OS.close();
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Save Error for " + FileName, 480,48, X.toString());
					return;
				}
			}}
		);
		C.gridy = (Row++);
		add(SaveButton,C);
		
		// Quit editing this file
		Button CloseButton = new Button("Close");
		CloseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {CloseAction();}}
		);
		C.gridy = (Row++);
		add(CloseButton,C);
		validate();
		setVisible(true);
		
		// Handle the close box:
		addWindowListener(new WindowListener()
			{
			public void windowOpened(WindowEvent E) {}
			public void windowClosing(WindowEvent E) {CloseAction();}
			public void windowClosed(WindowEvent E) {}
			public void windowIconified(WindowEvent E) {}
			public void windowDeiconified(WindowEvent E) {}
			public void windowActivated(WindowEvent E) {}
			public void windowDeactivated(WindowEvent E) {}
			}
		);
		
		setSize(getPreferredSize());
		validate();
	}
}
