// Level-data editor; is a panel in the level-vector editor

package tr_script;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import util.*;


final class LevelEditor extends Panel {
	
	// These store the level data to be edited
	private TextField NameTF;
	private TextField[] KeyTFList;
	private TextField FileTF;
	private Checkbox IfDemoCB;
	private Vector LevelScript; // Purely internal to avoid making spurious cross-references
	
	private Button EditScriptButton; // Up here so one can indicate whether the level script
	// has been edited or not.
	
	private Frame Parent;
	
	// Copy contents from one vector to another, assuming that the vectors
	// contain Short objects -- copy as a short integer.
	private void CopyShorts(Vector SrcLS, Vector DestLS) {
		int SSize = SrcLS.size();
		DestLS.removeAllElements();
		DestLS.setSize(SSize);
		for (int is=0; is<SSize; is++) {
			Short V = (Short)SrcLS.elementAt(is);
			short v = V.shortValue();
			DestLS.setElementAt(new Short(v),is);
		}
	}
	
	// Sets the level-data object's contents to this object's contents
	void SetLevelData(LevelData Level) {
		Level.Name = NameTF.getText();
		for (int ik=0; ik<LevelData.NumKeys; ik++)
			Level.KeyNames[ik] = KeyTFList[ik].getText();
		Level.FileName = FileTF.getText();
		Level.IfDemo = IfDemoCB.getState();
		CopyShorts(LevelScript,Level.Script);
		
		EditScriptButton.setLabel("Edit Script");
	}
	
	// Sets this object's contents to the level-data object's contents
	void GetLevelData(LevelData Level) {
		NameTF.setText(Level.Name);
		for (int ik=0; ik<LevelData.NumKeys; ik++)
			KeyTFList[ik].setText(Level.KeyNames[ik]);
		FileTF.setText(Level.FileName);
		IfDemoCB.setState(Level.IfDemo);
		CopyShorts(Level.Script,LevelScript);
		
		EditScriptButton.setLabel("Edit Script");
	}
	
	// Blanks it out
	void BlankData() {
		NameTF.setText("New Name");
		KeyTFList[0].setText("P1");
		KeyTFList[1].setText("P2");
		KeyTFList[2].setText("P3");
		KeyTFList[3].setText("P4");
		KeyTFList[4].setText("P1");
		KeyTFList[5].setText("P2");
		KeyTFList[6].setText("K1");
		KeyTFList[7].setText("K2");
		KeyTFList[8].setText("K3");
		KeyTFList[9].setText("K4");
		FileTF.setText("New Filename");
		IfDemoCB.setState(false);
		LevelScript.removeAllElements();
	}
	
	// Returns a level with the level data set in it
	LevelData MakeLevel() {
		LevelData Level = new LevelData();
		SetLevelData(Level);
		return Level;
	}
	
	LevelEditor(Frame _Parent) {
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
		
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// Necessary for some of the inner classes
		Parent = _Parent;
				
		// The text
		C.anchor = GridBagConstraints.WEST;
		
		KeyTFList = new TextField[LevelData.NumKeys];
		
		int Row = 0;
		
		C.gridx = 0;
		C.gridy = Row;
		add(new Label("Name: "),C);
		
		NameTF = new TextField(32);
		C.gridx = 1;
		add(NameTF,C);
		Row++;
				
		String[] KeyLabels = {"Puzzle 1: ","Puzzle 2: ","Puzzle 3: ","Puzzle 4: ",
			"Pickup 1: ","Pickup 2: ","Key 1: ","Key 2: ","Key 3: ","Key 4: "};
		for (int KeyNo = 0; KeyNo < LevelData.NumKeys; KeyNo++) {			
			C.gridx = 0;
			C.gridy = Row;
			add(new Label(KeyLabels[KeyNo]),C);
			
			KeyTFList[KeyNo] = new TextField(32);
			C.gridx = 1;
			add(KeyTFList[KeyNo],C);
			Row++;
		}
		
		C.gridx = 0;
		C.gridy = Row;
		add(new Label("File: "),C);
		
		FileTF = new TextField(32);
		C.gridx = 1;
		add(FileTF,C);
		Row++;
		
		IfDemoCB = new Checkbox("Is a Demo Level");
		C.gridx = 1;
		C.gridy = (Row++);
		add(IfDemoCB,C);
		
		// The buttons
		C.fill = GridBagConstraints.HORIZONTAL;
		
		// Script editing
		LevelScript = new Vector();
		EditScriptButton = new Button("Edit Script");
		EditScriptButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				LevelScriptEditor LSE =
					new LevelScriptEditor(Parent,LevelScript,"\"" + NameTF.getText() + "\" Script");
				LSE.dispose();
				EditScriptButton.setLabel("Edit Script (Has Been Edited)");
			}}
		);
		C.gridy = (Row++);
		add(EditScriptButton,C);
	}
}
