// This is a dialog box for editing level scripts
// Set up not to dispose of itself when done, to avoid crashes in MW Java / MRJ

package tr_script;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import util.*;

final class LevelScriptEditor extends Dialog {
	
	// Stuff that has to be local
	private String Name;
	private Frame Parent;
	private Vector ScriptVec;
	
	private List DispList;
	
	// Parsed List, for cooked-mode editing
	private Vector ParsedScriptVec;
	
	// Note: EditField has an action routine attached
	// that continually updates EditValue
	private TextField EditField;
	private short EditValue;
	
	private TextArea ParsedScriptTA;
	
	// Write out some parsing;
	// depends on how many operands there are per opcode
	
	private void NoOperands(StringBuffer ST, Short Opcode, String Descr) {
		ST.append(Opcode);
		ST.append("\n");
		ST.append(Descr);
		ST.append("\n");
	}
	
	private void OneOperand(StringBuffer ST, Short Opcode, Short Operand, String Descr) {
		ST.append(Opcode);
		ST.append(" ");
		ST.append(Operand);
		ST.append("\n");
		ST.append(Descr);
		ST.append(Operand);
		ST.append("\n");
	}
	
	// Do the parsing
	private void DoParse() {
		StringBuffer ScriptText = new StringBuffer("");
		int indx = 0;
		boolean CorrectlyParsed = true;
		boolean CorrectlyEnded = false;
		ParsedScriptVec = new Vector();
		
		while(indx < ScriptVec.size()) {
			Short Opcode = (Short)ScriptVec.elementAt(indx++);
			Short Operand;
			short OpVal = Opcode.shortValue();
			LevelScriptOp NewOp = new LevelScriptOp();
			NewOp.Opcode = OpVal;
			
			LevelScriptParseInfo ParseInfo = LevelScriptParseInfo.Do(OpVal);
			if (ParseInfo.NumOperands >= 1) {
				if (indx >= ScriptVec.size()) {
					CorrectlyParsed = false;
					NoOperands(ScriptText, Opcode, ParseInfo.Description + ": needs an operand");
				} else {
					Operand = (Short)ScriptVec.elementAt(indx++);
					NewOp.Operand = Operand.shortValue();
					OneOperand(ScriptText, Opcode, Operand, ParseInfo.Description + ": ");
				}
			} else {
				NoOperands(ScriptText, Opcode, ParseInfo.Description);
			}
			if (OpVal == 9) CorrectlyEnded = true;
			ParsedScriptVec.addElement(NewOp);
			
			if (CorrectlyEnded) break;
		}
		
		if (!CorrectlyEnded) {
			CorrectlyParsed = false;
			ScriptText.append("*** No Ending Code ***\n");
		}
		
		if (!CorrectlyParsed) ParsedScriptVec = null;
		
		while(indx < ScriptVec.size()) {
			ScriptText.append(ScriptVec.elementAt(indx));
			ScriptText.append(" ");
			indx++;
		}
		
		ParsedScriptTA.setText(ScriptText.toString());
	}
	
	// Create a string in a suitable format for the display list:
	private String MakeDispString(int ic, short Value) {
		return (Integer.toString(ic) + ": " + Short.toString(Value));
	}
	
	// Rebuild the display list
	private void MakeDispList() {
		DispList.removeAll();
		for (int ic=0; ic<ScriptVec.size(); ic++) {
			Short Value = (Short)ScriptVec.elementAt(ic);
			DispList.add(MakeDispString(ic,Value.shortValue()));
		}
	}	
	
	LevelScriptEditor(Frame _Parent, Vector _ScriptVec, String _Name) {
		super(_Parent,_Name,true);
		setSize(640,384);
		setResizable(false);
		
		// Necessary for some of the inner classes
		Name = _Name;
		Parent = _Parent;
		ScriptVec = _ScriptVec;
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
				
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// The display list itself
		DispList = new List(16);
		MakeDispList();
		
		// Action is to get a string from the list
		DispList.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				Short Value = (Short)ScriptVec.elementAt(ic);
				EditValue = Value.shortValue();
				EditField.setText(Short.toString(EditValue));
			}}
		);
		
		C.gridx = 0;
		C.gridheight = 12;
		C.anchor = GridBagConstraints.NORTHWEST;
		add(DispList,C);
		C.gridheight = 1;
		
		// The edit-text field
		EditField = new TextField("0",24);
		EditField.addTextListener(new TextListener()
			{public void textValueChanged(TextEvent E) {
				try {
					EditValue = Short.parseShort(EditField.getText());
				} catch(Exception X) {
					EditField.setText(Short.toString(EditValue));
				}
			}}
		);
		int Row = 0;
		C.gridx = 1;
		C.gridy = (Row++);
		C.anchor = GridBagConstraints.WEST;
		add(EditField,C);
		
		// The buttons:
		C.fill = GridBagConstraints.HORIZONTAL;
		C.anchor = GridBagConstraints.WEST;

		// Parse the current script
		Button ParseButton = new Button("Parse");
		ParseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				DoParse();
			}}
		);
		C.gridy = (Row++);
		add(ParseButton,C);

		// Edit the current script in cooked mode
		Button EditCookedButton = new Button("Edit Cooked");
		EditCookedButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				DoParse();
				if (ParsedScriptVec != null) {
					CookedLevelScriptEditor CLSE =
						new CookedLevelScriptEditor(Parent,ParsedScriptVec,Name + " (Cooked)");
					CLSE.dispose();
					ScriptVec.removeAllElements();
					for (int k=0; k<ParsedScriptVec.size(); k++) {
						LevelScriptOp Op = (LevelScriptOp)ParsedScriptVec.elementAt(k);
						short OpVal = Op.Opcode;
						ScriptVec.addElement(new Short(OpVal));
						LevelScriptParseInfo ParseInfo = LevelScriptParseInfo.Do(OpVal);
						if (ParseInfo.NumOperands >= 1)
							ScriptVec.addElement(new Short(Op.Operand));
					}
					MakeDispList();
					DoParse();
				} else {
					InfoAlert A =
						new InfoAlert(Parent, "Error in Level-Script Editing", 480,48,
							"Cooked editing could not be done because the script parsing is erroneous");
				}
			}}
		);
		C.gridy = (Row++);
		add(EditCookedButton,C);
		
		// Get the currently-selected value
		Button GetButton = new Button("Get from List");
		GetButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				Short Value = (Short)ScriptVec.elementAt(ic);
				EditValue = Value.shortValue();
				EditField.setText(Short.toString(EditValue));
			}}
		);
		C.gridy = (Row++);
		add(GetButton,C);
		
		// Set the currently-selected value to the text-field contents
		Button ChangeButton = new Button("Change in List");
		ChangeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				ScriptVec.setElementAt(new Short(EditValue),ic);
				DispList.replaceItem(MakeDispString(ic,EditValue),ic);
					
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(ChangeButton,C);
		
		// Insert a value before the currently-selected value location
		Button InsertBeforeButton = new Button("Insert Before");
		InsertBeforeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (ScriptVec.size() <= 0) {
					ScriptVec.addElement(new Short(EditValue));
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					ScriptVec.insertElementAt(new Short(EditValue),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertBeforeButton,C);
		
		// Insert a value after the currently-selected value location
		Button InsertAfterButton = new Button("Insert After");
		InsertAfterButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (ScriptVec.size() <= 0) {
					ScriptVec.addElement(new Short(EditValue));
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					ic++;
					ScriptVec.insertElementAt(new Short(EditValue),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertAfterButton,C);

		// Delete the currently-selected string
		Button DeleteButton = new Button("Delete");
		DeleteButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				WarningAlert A = new WarningAlert(Parent,"Warning!",384,72,
					"Really want to delete \"" + (Short)ScriptVec.elementAt(ic) + "\"?");
				if (A.If_OK) {
					ScriptVec.removeElementAt(ic);
					MakeDispList();
					DispList.select(ic);
				
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(DeleteButton,C);
		
		// Load the strings from a file
		Button LoadButton = new Button("Load Script");
		LoadButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Load list from file",FileDialog.LOAD);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File InFile = new File(DirName,FileName);
				
				FileReader FR;
				
				ScriptVec.removeAllElements();
				
				try {
					FR = new FileReader(InFile);
					
					// So I don't have to do this by hand (heh heh)
					StreamTokenizer Grabber = new StreamTokenizer(FR);
					Grabber.resetSyntax();
					Grabber.wordChars(0,255);
					Grabber.whitespaceChars('\n','\n');
					Grabber.whitespaceChars('\r','\r');
					Grabber.whitespaceChars('\t','\t');
					Grabber.whitespaceChars(' ',' ');
					
					while(true) {
						int TokenType = Grabber.nextToken();
						if (TokenType == StreamTokenizer.TT_WORD) {
							String ReadString = Grabber.sval;
							if (ReadString.length() != 0)
								ScriptVec.addElement(new Short(ReadString));
						} else if (TokenType == StreamTokenizer.TT_EOF) break;
					}
					FR.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Load Error for " + FileName, 480,48, X.toString());
				} finally {					
					MakeDispList();
				
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(LoadButton,C);
		
		// Save the strings to a file
		Button SaveButton = new Button("Save Script");
		SaveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Save list in file",FileDialog.SAVE);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File OutFile = new File(DirName,FileName);
				
				FileWriter FW;
				
				try {
					FW = new FileWriter(OutFile);
				
					for (int ic=0; ic<ScriptVec.size(); ic++) {
						FW.write(((Short)ScriptVec.elementAt(ic)).toString());
						FW.write(System.getProperty("line.separator"));
					}
					FW.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Save Error for " + FileName, 480,48, X.toString());
					return;
				}
			}}
		);
		C.gridy = (Row++);
		add(SaveButton,C);
		
		Button CloseButton = new Button("Close");
		CloseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E)
				{/*dispose();*/setVisible(false);
			}}
		);
		C.gridy = (Row++);
		add(CloseButton,C);
		
		// The parsed script
		ParsedScriptTA = new TextArea(12,32);
		ParsedScriptTA.setEditable(false);
		C.gridheight = ++Row;
		C.fill = GridBagConstraints.NONE;
		C.anchor = GridBagConstraints.NORTHWEST;
		C.gridx = 2;
		C.gridy = 0;
		add(ParsedScriptTA,C);
		C.gridheight = 1;
		
		DoParse();
						
		validate();
		setVisible(true);
		
		setSize(getPreferredSize());
		validate();
	}
}
