// This class displays parsing information for level scripts.
// More specifically, it takes an opcode and returns various parsing info.

package tr_script;

final class LevelScriptParseInfo {
	// Parsing info returned
	String Description;
	short NumOperands;
	
	// Opcode bounds
	static short MinOpcode() {return 3;}
	static short MaxOpcode() {return 22;}
	
	// Constructs a parse-info structure from an opcode
	static LevelScriptParseInfo Do(short Opcode) {
		LevelScriptParseInfo ParseInfo = new LevelScriptParseInfo();
		
		switch(Opcode) {
		case 3:
			ParseInfo.Description = "Play FMV";
			ParseInfo.NumOperands = 1;
			break;
		
		case 4:
			ParseInfo.Description = "Play interactive level";
			ParseInfo.NumOperands = 1;
			break;
			
		case 5:
			ParseInfo.Description = "Play cutscene";
			ParseInfo.NumOperands = 1;
			break;
			
		case 6:
			ParseInfo.Description = "Level-completion display";
			ParseInfo.NumOperands = 0;
			break;
			
		case 7:
			ParseInfo.Description = "Play demo level";
			ParseInfo.NumOperands = 1;
			break;
			
		case 9:
			ParseInfo.Description = "End of script";
			ParseInfo.NumOperands = 0;
			break;
			
		case 10:
			ParseInfo.Description = "Play soundtrack";
			ParseInfo.NumOperands = 1;
			break;
			
		case 11:
			ParseInfo.Description = "Unknown opcode: #11";
			ParseInfo.NumOperands = 0;
			break;
			
		case 12:
			ParseInfo.Description = "Make chapter screen";
			ParseInfo.NumOperands = 1;
			break;
			
		case 13:
			ParseInfo.Description = "Unknown opcode: #13";
			ParseInfo.NumOperands = 0;
			break;
			
		case 14:
			ParseInfo.Description = "Take away weapons";
			ParseInfo.NumOperands = 0;
			break;
			
		case 15:
			ParseInfo.Description = "End of game";
			ParseInfo.NumOperands = 0;
			break;
			
		case 16:
			ParseInfo.Description = "Cutscene angle";
			ParseInfo.NumOperands = 1;
			break;
			
		case 17:
			ParseInfo.Description = "Unknown opcode: #17";
			ParseInfo.NumOperands = 1;
			break;
			
		case 18:
			ParseInfo.Description = "Give item";
			ParseInfo.NumOperands = 1;
			break;
			
		case 19:
			ParseInfo.Description = "Starting animation";
			ParseInfo.NumOperands = 1;
			break;
			
		case 20:
			ParseInfo.Description = "Number of secrets";
			ParseInfo.NumOperands = 1;
			break;
			
		case 21:
			ParseInfo.Description = "Kill enemies to finish level";
			ParseInfo.NumOperands = 0;
			break;
			
		case 22:
			ParseInfo.Description = "Take away ammo";
			ParseInfo.NumOperands = 0;
			break;
			
		default:
			// Indicate something that was not parsed
			ParseInfo = null;
			break;
		}
		
		return ParseInfo;
	}
}
