// Level-vector editor

package tr_script;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import util.*;


final class LevelVecEditor extends Dialog {
	
	// Stuff that has to be local
	private Frame Parent;
	private Vector LevelVec;
	
	// Displayed list of strings:
	private List DispList;
	
	// Panel for level contents:
	private LevelEditor LE;
		
	// Create a string in a suitable format for the display list:
	private String MakeDispString(int ic, LevelData Level) {
		return (Integer.toString(ic) + ": " + Level.Name);
	}
	
	// Rebuild the display list
	private void MakeDispList() {
		DispList.removeAll();
		for (int ic=0; ic<LevelVec.size(); ic++) {
			LevelData Level = (LevelData)LevelVec.elementAt(ic);
			DispList.add(MakeDispString(ic,Level));
		}
	}
		
	LevelVecEditor(Frame _Parent, Vector _LevelVec, String Name) {
		super(_Parent,Name);
		setSize(512,384);
		setResizable(false);
		
		// Necessary for some of the inner classes
		Parent = _Parent;
		LevelVec = _LevelVec;
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
		
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// The display list itself
		DispList = new List(16);
		MakeDispList();
		
		// Action is to get a level from the list
		DispList.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				LevelData Level = (LevelData)LevelVec.elementAt(ic);
				LE.GetLevelData(Level);
			}}
		);
		
		C.gridx = 0;
		C.gridheight = 16;
		C.anchor = GridBagConstraints.NORTHWEST;
		add(DispList,C);
		C.gridheight = 1;
		
		// The control buttons
		
		// Blank out the level workspace
		Button BlankButton = new Button("Blank Out");
		BlankButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				LE.BlankData();
			}}
		);
		int Row = 0;
		C.fill = GridBagConstraints.HORIZONTAL;
		C.anchor = GridBagConstraints.WEST;
		C.gridx = 1;
		C.gridy = (Row++);
		add(BlankButton,C);
		
		// Get the currently-selected level
		Button GetButton = new Button("Get from List");
		GetButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				LevelData Level = (LevelData)LevelVec.elementAt(ic);
				LE.GetLevelData(Level);
			}}
		);
		C.gridy = (Row++);
		add(GetButton,C);
		
		// Set the currently-selected level to the level-workspace contents
		Button ChangeButton = new Button("Change in List");
		ChangeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				LevelData Level = (LevelData)LevelVec.elementAt(ic);
				LE.SetLevelData(Level);
				DispList.replaceItem(MakeDispString(ic,Level),ic);
					
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(ChangeButton,C);

		// Duplicate the current level
		Button DuplicateButton = new Button("Duplicate");
		DuplicateButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				LevelData Level = new LevelData((LevelData)LevelVec.elementAt(ic));
				LevelVec.insertElementAt(Level,ic);
				MakeDispList();
				DispList.select(ic);
			}}
		);
		C.gridy = (Row++);
		add(DuplicateButton,C);

		// Move the current level to some location
		Button MoveButton = new Button("Move");
		MoveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				int LVLen = LevelVec.size();
				String[] LocList = new String[LVLen + 1];
				for (int ix=0; ix<LVLen; ix++) {
					LevelData Level = (LevelData)LevelVec.elementAt(ix);
					LocList[ix] = Level.Name;
				}
				LocList[LVLen] = "<<< End >>>";
				LevelData Level = (LevelData)LevelVec.elementAt(ic);
				StringLocSelector SLE = 
					new StringLocSelector(Parent,"Level-List Editor",
					"Move string \"" + Level.Name + "\" to before",LocList);
				if (SLE.DoGoto) {
					// This bumps down all strings after it
					LevelVec.removeElementAt(ic);
					int Loc = SLE.Loc;
					if (Loc > ic) Loc--;
					LevelVec.insertElementAt(Level,Loc);
					MakeDispList();
					DispList.select(Loc);
				}
			}}
		);
		C.gridy = (Row++);
		add(MoveButton,C);

		// Insert the workspace level before the currently-selected string location
		Button InsertBeforeButton = new Button("Insert Before");
		InsertBeforeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (LevelVec.size() <= 0) {
					LevelVec.addElement(LE.MakeLevel());
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					LevelVec.insertElementAt(LE.MakeLevel(),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertBeforeButton,C);
		
		// Insert the workspace level after the currently-selected string location
		Button InsertAfterButton = new Button("Insert After");
		InsertAfterButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (LevelVec.size() <= 0) {
					LevelVec.addElement(LE.MakeLevel());
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					ic++;
					LevelVec.insertElementAt(LE.MakeLevel(),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertAfterButton,C);

		// Delete the currently-selected level
		Button DeleteButton = new Button("Delete");
		DeleteButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				LevelData Level = (LevelData)LevelVec.elementAt(ic);
				WarningAlert A = new WarningAlert(Parent,"Warning!",384,72,
					"Really want to delete \"" + Level.Name + "\"?");
				if (A.If_OK) {
					LevelVec.removeElementAt(ic);
					MakeDispList();
					DispList.select(ic);
				
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(DeleteButton,C);

		// Load the level names from a file
		Button LoadButton = new Button("Load Names");
		LoadButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Load names from file",FileDialog.LOAD);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File InFile = new File(DirName,FileName);
				
				FileReader FR;
				
				try {
					FR = new FileReader(InFile);
					
					// So I don't have to do this by hand (heh heh)
					StreamTokenizer Grabber = new StreamTokenizer(FR);
					Grabber.resetSyntax();
					Grabber.wordChars(0,255);
					Grabber.whitespaceChars('\n','\n');
					Grabber.whitespaceChars('\r','\r');
					
					int ic = 0, ik = -1;
					LevelData Level = new LevelData();
					while(ic < LevelVec.size()) {
						int TokenType = Grabber.nextToken();
						if (TokenType == StreamTokenizer.TT_WORD) {
							String ReadString = Grabber.sval;
							if (ReadString.length() != 0) {
								if (ik == -1) {
									Level = (LevelData)LevelVec.elementAt(ic);
									Level.Name = ReadString;
								} else {
									Level.KeyNames[ik] = ReadString;
								}
								if ((++ik) >= LevelData.NumKeys) {
									ik = -1;
									ic++;
								}
							}
						} else if (TokenType == StreamTokenizer.TT_EOF) break;
					}
					FR.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Load Error for " + FileName, 480,48, X.toString());
					return;
				} finally {
					MakeDispList();
				
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(LoadButton,C);
		
		// Save the level names to a file
		Button SaveButton = new Button("Save Names");
		SaveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Save names in file",FileDialog.SAVE);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File OutFile = new File(DirName,FileName);
				
				FileWriter FW;
				
				try {
					FW = new FileWriter(OutFile);
				
					for (int ic=0; ic<LevelVec.size(); ic++) {
						LevelData Level = (LevelData)LevelVec.elementAt(ic);
						FW.write(Level.Name);
						FW.write(System.getProperty("line.separator"));
						for (int ik=0; ik<LevelData.NumKeys; ik++) {
							FW.write(Level.KeyNames[ik]);
							FW.write(System.getProperty("line.separator"));
						}
						FW.write(System.getProperty("line.separator"));							
					}
					FW.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Save Error for " + FileName, 480,48, X.toString());
					return;
				}
			}}
		);
		C.gridy = (Row++);
		add(SaveButton,C);
		
		// Close this dialog box (changes are saved automatically)
		Button CloseButton = new Button("Close");
		CloseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {dispose();}}
		);
		C.gridy = (Row++);
		add(CloseButton,C);

		// Handle the close box:
		addWindowListener(new WindowListener()
			{
			public void windowOpened(WindowEvent E) {}
			public void windowClosing(WindowEvent E) {dispose();}
			public void windowClosed(WindowEvent E) {}
			public void windowIconified(WindowEvent E) {}
			public void windowDeiconified(WindowEvent E) {}
			public void windowActivated(WindowEvent E) {}
			public void windowDeactivated(WindowEvent E) {}
			}
		);
		
		// Level-editor panel:
		LE = new LevelEditor(Parent);
		
		C.gridx = 2;
		C.gridy = 0;
		C.gridheight = 16;
		C.anchor = GridBagConstraints.NORTHWEST;
		add(LE,C);
		C.gridheight = 1;

		validate();
		setVisible(true);
		
		setSize(getPreferredSize());
		validate();
	}
}
