/*

	This class is the master-dialog class
	
*/

package tr_script;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import util.*;


public final class MasterDialog extends Frame {
	
	//  Make the open directory and filename persistent
	private String DirName, FileName;
	
	// Whether the script file is in PSX flavor
	boolean IsPSX;
	
	// Which numbers of strings in game-string-set 2:
	static final short NumGameStrings2_PC = 0x29;
	static final short NumGameStrings2_PSX = 0x50;
	
	// Open-file routine
	private void OpenFile() {
		FileDialog FD = new FileDialog(this,"Load TOMBPC.DAT file",FileDialog.LOAD);
		FD.setDirectory(DirName);
		FD.setFile(FileName);
		FD.setVisible(true);
		String DirName = FD.getDirectory();
		String FileName = FD.getFile();
		if (FileName == null) return;
		File InFile = new File(DirName,FileName);
		
		InputStream IS;
		ScriptData SD;
		
		try {
			IS = (InputStream)new FileInputStream(InFile);
			SD = new ScriptData();
			SD.IsPSX = IsPSX;
			SD.NumGameStrings2 = IsPSX ? NumGameStrings2_PSX : NumGameStrings2_PC;
			SD.FileName = FileName;
			if (!ScriptIO.Read(this, IS, SD)) return;
			IS.close();
		} catch (Exception X) {
			InfoAlert A =
				new InfoAlert(this, "Open Error for " + FileName, 480,48, X.toString());
			return;
		}
		
		ScriptEditor SE = new ScriptEditor(this,SD);
	}
	
	// For the Quit warning alert
	private Frame Parent;

	public MasterDialog() {
		setTitle("TRScripter Master Dialog");
		setSize(384,64);
		setResizable(false);
		
		// Hack to get the Quit warning alert to use this frame
		Parent = this;
		
		// Create the menu:
		
		MenuItem OpenItem = new MenuItem("Open...");
		OpenItem.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {OpenFile();}}
		);
		
		// Initial setting: PC/Mac
		IsPSX = false;
		
		final CheckboxMenuItem FlavorItem_PC = new CheckboxMenuItem("PC/Mac",!IsPSX);
		final CheckboxMenuItem FlavorItem_PSX = new CheckboxMenuItem("Playstation",IsPSX);
		
		FlavorItem_PC.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FlavorItem_PC.setState(true);
				FlavorItem_PSX.setState(false);
				IsPSX = false;
			}}
		);
		FlavorItem_PSX.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FlavorItem_PC.setState(false);
				FlavorItem_PSX.setState(true);
				IsPSX = true;
			}}
		);
		Menu FlavorMenu = new Menu("Flavor");
		FlavorMenu.add(FlavorItem_PC);
		FlavorMenu.add(FlavorItem_PSX);
		
		MenuItem QuitItem = new MenuItem("Quit");
		QuitItem.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				WarningAlert A = new WarningAlert(Parent,"Warning!",384,72,
					"Really want to quit TRScripter?");
				if (A.If_OK) {
					dispose();
					System.exit(0);
				}
			}}
		);
		
		Menu FileMenu = new Menu("File");
		FileMenu.add(OpenItem);
		FileMenu.add(FlavorMenu);
		FileMenu.add(QuitItem);
		
		MenuBar MB = new MenuBar();
		MB.add(FileMenu);
		setMenuBar(MB);
		
		setLayout(new GridBagLayout());
		add(new Label("TRScripter, an Editor for Tomb Raider Game Scripts"));
				
		validate();
		setVisible(true);
	}
};