/*

	These objects contain all the tombpc.dat TR-script data;
	all the per-level data is in the object LevelData

*/


package tr_script;

import java.util.Vector;


final class ScriptData {
	
	String FileName;		// Its file name
	
	int	Version;			// Version number
	String Info;			// Info string
	
	static final int VersionValue = 3;	// Canonical version value
	
	int FirstOption;		// Level to go to when that happens (1280 is exit-to-title)
    int TitleReplace; 		// Level to go to when that happens (-1 is NONE)
    int OnDeathDemoMode;	// Level to go to when that happens (1280 is exit-to-title)
    int OnDeathInGame;		// Level to go to when that happens (0 is exit-to-title)
    int DemoTime;			// Time in game ticks to wait before starting a demo
    int OnDemoInterrupt;	// Level to go to when that happens (1280 is exit-to-title)
    int OnDemoEnd;			// Level to go to when that happens (1280 is exit-to-title)
	
	// Unused1 goes here
	
	// Keep these in sync with the arrays later on
    short NumLevels;		// Number of levels in game (some level files repeated)
    short NumChapScreens;	// Number of chapter screens
    short NumTitles;		// Number of title displays
    short NumFMVs;			// Number of prerendered cutscenes (FMV's)
    short NumCutscenes;		// Number of engine-rendered cutscenes
    short NumDemoLevels;	// Number of demo levels
    
    short TitleSoundID;		// ID of title soundtrack
    short SingleLevel;		// Which single level to do (-1 is NONE)
	
	// Unused2 goes here
	
    short Flags;			// Various flags
	
	// Unused3 goes here
	
    byte XORByte;			// XOR encryption byte
	
	byte BowdlerByte;		// Blood-bowdlerization byte
	
    short SecretSoundID;	// ID of found-a-secret soundtrack
	
	// Unused5 goes here
	
	// A variable number of these (type is String):
	Vector Titles;			// Title files
    Vector ChapScreens;		// Static chapter screens
    Vector FMVs;			// Prerendered cutscenes (FMV's)
    Vector Cutscenes;		// Engine-rendered cutscenes
    
    // Get this from Levels[]
    short[] DemoLevels;		// ID's of demo levels
    
    // Extra block of data in Playstation version, but not PC/Mac version;
    // this data may be offsets to something FMV-related
    boolean IsPSX;
    int[] ExtraPSXData;
    
    // The first set may have a variable number,
    // but the second set has a fixed number
    String[] GameStrings1;	// Game strings #1
    String[] GameStrings2;	// Game strings #2
	
	Vector StartupScript;	// Level script for starting the game
							// (list of shorts stored in Short wrapper objects)
	
	// The members have type LevelData
    Vector Levels;			// Everything associated with each level
	
	// How many of these items;
	// the number of set-2 ones is dependent on which flavor of file
	static short NumGameStrings1;
	static short NumGameStrings2;
    
    // Flag values:
    static final short Flag_DemoVersion =				0x0001;
    static final short Flag_Title_Disabled =			0x0002;
    static final short Flag_CheatModeCheck_Disabled =	0x0004;
    static final short Flag_NoInput_Timeout =			0x0008;
    
    static final short Flag_LoadSave_Disabled =			0x0010;
    static final short Flag_Screensizing_Disabled =		0x0020;
    static final short Flag_Lockout_OptionRing =		0x0040;
    static final short Flag_DOZY_Cheat_Enabled =		0x0080;
    
    static final short Flag_Use_Encryption =			0x0100;
    static final short Flag_Unknown09 =					0x0200;
    static final short Flag_Select_Any_Level =			0x0400;
    static final short Flag_Unknown11 =					0x0800;
    
    static final short Flag_Unknown12 =					0x1000;
    static final short Flag_Unknown13 =					0x2000;
    static final short Flag_Unknown14 =					0x4000;
    static final short Flag_Unknown15 =					(short)0x8000;
    
    // These are usually 0, but I'll keep them around anyway
    byte[] Unused1;
    byte[] Unused2;
    byte[] Unused3;
    byte[] Unused5;
	
	// Expected numbers of bytes:
	static final int InfoLen = 0x100;
	static final int HeaderLen = 0x80;
	static final int Unused1Len = 0x24;
	static final int Unused2Len = 0x20;
	static final int Unused3Len = 0x06;
	static final int Unused5Len = 0x04;
	
	// This updates the header values and list of demo levels,
	// so they will be in sync with the quantities actually edited
	void Synchronize() {
		NumLevels = (short)Levels.size();
		NumChapScreens = (short)ChapScreens.size();
		NumTitles = (short)Titles.size();
		NumFMVs = (short)FMVs.size();
		NumCutscenes = (short)Cutscenes.size();
		
		NumDemoLevels = (short)0;
		for (int n=0; n<NumLevels; n++) {
			LevelData L = (LevelData)Levels.elementAt(n);
			if (L.IfDemo) NumDemoLevels++;
		}
		DemoLevels = new short[NumDemoLevels];
		int IndxDemoLevels = 0;
		for (int n=0; n<NumLevels; n++) {
			LevelData L = (LevelData)Levels.elementAt(n);
			if (L.IfDemo) DemoLevels[IndxDemoLevels++] = (short)n;
		}
		
		NumGameStrings1 = (short)GameStrings1.length;
	}
}
