/*

	This is the script-editor master object, which contains the script data and
	which contains the master dialog box for editing
	
*/

package tr_script;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import util.*;


// It's a subclass of Dialog since a dialog box is an intimate part of the editor
final class ScriptEditor extends Dialog {

	// Script data to be edited
	ScriptData SD;
	
	// Info text field
	TextArea InfoField;
	
	// Numerical fields
	CaptionedNumberField FirstOptionField;			// Level to go to when that happens (1280 is exit-to-title)
    CaptionedNumberField TitleReplaceField; 		// Level to go to when that happens (-1 is NONE)
    CaptionedNumberField OnDeathDemoModeField;		// Level to go to when that happens (1280 is exit-to-title)
    CaptionedNumberField OnDeathInGameField;		// Level to go to when that happens (0 is exit-to-title)
    CaptionedNumberField DemoTimeField;				// Time in game ticks to wait before starting a demo
    CaptionedNumberField OnDemoInterruptField;		// Level to go to when that happens (1280 is exit-to-title)
    CaptionedNumberField OnDemoEndField;			// Level to go to when that happens (1280 is exit-to-title)
    CaptionedNumberField SingleLevelField;			// Which single level to do (-1 is NONE)
    CaptionedNumberField TitleSoundIDField;			// ID of title soundtrack
    CaptionedNumberField SecretSoundIDField;		// ID of found-a-secret soundtrack
    CaptionedNumberField XORByteField;				// XOR encryption byte
    CaptionedNumberField BowdlerByteField;			// Blood-bowdlerization byte
    
    // Flag checkboxes
    FlagCheckBoxSet FCBS;
    
	// Update the script data
	void UpdateScriptData() {
		SD.Info = InfoField.getText();
		
		SD.FirstOption = FirstOptionField.GetValue();
    	SD.TitleReplace = TitleReplaceField.GetValue();
    	SD.OnDeathDemoMode = OnDeathDemoModeField.GetValue();
    	SD.OnDeathInGame = OnDeathInGameField.GetValue();
    	SD.DemoTime = DemoTimeField.GetValue();
    	SD.OnDemoInterrupt = OnDemoInterruptField.GetValue();
    	SD.OnDemoEnd = OnDemoEndField.GetValue();
    	SD.SingleLevel = (short)SingleLevelField.GetValue();
    	SD.TitleSoundID = (short)TitleSoundIDField.GetValue();
    	SD.SecretSoundID = (short)SecretSoundIDField.GetValue();
    	SD.XORByte = (byte)XORByteField.GetValue();
     	SD.BowdlerByte = (byte)BowdlerByteField.GetValue();
   	
    	SD.Flags = FCBS.GetValue();
 	}
	
	// Parent frame
	private Frame Parent;
	
	// Make the save directory and filename persistent
	private String DirName, FileName;
	
	// Close-window action:
	private void CloseAction() {
		WarningAlert A = new WarningAlert(Parent,"Warning!",384,72,
			"Really want to stop editing " + SD.FileName + "?");
		if (A.If_OK) dispose();
	}
	
	// Needs a parent frame
	ScriptEditor(Frame _Parent, ScriptData _SD) {
		super(_Parent,_SD.FileName + " Editor");
		setResizable(false);
		setSize(640,512);
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
		
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// Plug in
		Parent = _Parent;
		SD = _SD;
		
		int Row = 0;
		
		// Information field (extra wide)
		C.gridwidth = 3;
		InfoField = new TextArea(SD.Info,2,72);
		C.gridy = (Row++);
		add(InfoField,C);
		C.gridwidth = 1;		

		// Numerical fields
		int NFWidth = 8;
		C.anchor = GridBagConstraints.WEST;
		
		FirstOptionField = new CaptionedNumberField(NFWidth,"First-Option Level",SD.FirstOption);
    	C.gridy = (Row++);
		add(FirstOptionField,C);		
		
		TitleReplaceField = new CaptionedNumberField(NFWidth,"Title-Replacement Level",SD.TitleReplace);
    	C.gridy = (Row++);
		add(TitleReplaceField,C);		

		OnDeathDemoModeField = new CaptionedNumberField(NFWidth,"Death-in-Demo-Mode Level",SD.OnDeathDemoMode);
    	C.gridy = (Row++);
		add(OnDeathDemoModeField,C);		

		OnDeathInGameField = new CaptionedNumberField(NFWidth,"Death-in-Game Level",SD.OnDeathInGame);
    	C.gridy = (Row++);
		add(OnDeathInGameField,C);		

		DemoTimeField = new CaptionedNumberField(NFWidth,"Time to Demo Start",SD.DemoTime);
    	C.gridy = (Row++);
		add(DemoTimeField,C);		

		OnDemoInterruptField = new CaptionedNumberField(NFWidth,"Demo-Interrupt Level",SD.OnDemoInterrupt);
    	C.gridy = (Row++);
		add(OnDemoInterruptField,C);		

		OnDemoEndField = new CaptionedNumberField(NFWidth,"Demo-End Level",SD.OnDemoEnd);
     	C.gridy = (Row++);
		add(OnDemoEndField,C);
		
		SingleLevelField = new CaptionedNumberField(NFWidth,"Which Single Level",SD.SingleLevel);
     	C.gridy = (Row++);
		add(SingleLevelField,C);

		TitleSoundIDField = new CaptionedNumberField(NFWidth,"Title Sound ID",SD.TitleSoundID);
     	C.gridy = (Row++);
		add(TitleSoundIDField,C);

		SecretSoundIDField = new CaptionedNumberField(NFWidth,"Secret Sound ID",SD.SecretSoundID);
     	C.gridy = (Row++);
		add(SecretSoundIDField,C);

		XORByteField = new CaptionedNumberField(NFWidth,"Encryption Byte",+ SD.XORByte);
     	C.gridy = (Row++);
		add(XORByteField,C);

		BowdlerByteField = new CaptionedNumberField(NFWidth,"Blood-Bowdlerization Byte",+ SD.BowdlerByte);
     	C.gridy = (Row++);
		add(BowdlerByteField,C);
		
		// Flag captions
		String[] FlagCaptions = {
			"Demo Version",
			"Title Disabled",
			"Cheat-Mode Check Disabled",
			"No-Input Timeout",
			
			"Load/Save Disabled",
			"Screensizing Disabled",
			"Lock Out Option Ring",
			"DOZY Cheat Enabled",
			
			"Use Encryption",
			"Practice Level Enabled",
			"Select Any Level",
			"Unknown Bit: 11",
			
			"Unknown Bit: 12",
			"Unknown Bit: 13",
			"Unknown Bit: 14",
			"Unknown Bit: 15"
		};
		
		// Start new column for flag checkboxes
		C.gridheight = 12;
		C.anchor = GridBagConstraints.NORTHWEST;
     	C.gridy = 1;
     	FCBS = new FlagCheckBoxSet(FlagCaptions,SD.Flags);
		add(FCBS,C);
		C.gridheight = 1;
		 		
		// Start new column for the various control buttons
		C.fill = GridBagConstraints.HORIZONTAL;
		C.anchor = GridBagConstraints.WEST;
		C.gridx = 2;
		Row = 1;
		
		// Edit the level list
		Button EditLevelButton = new Button("Edit Levels");
		EditLevelButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				LevelVecEditor LVE =
					new LevelVecEditor(Parent,SD.Levels,SD.FileName + " Levels");
			}}
		);
		C.gridy = (Row++);
		add(EditLevelButton,C);
		
		// Edit the title files
		Button EditTitleButton = new Button("Edit Title Files");
		EditTitleButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				StringVecEditor SVE =
					new StringVecEditor(Parent,SD.Titles,SD.FileName + " Title Files");
			}}
		);
		C.gridy = (Row++);
		add(EditTitleButton,C);
		
		// Edit the chapter-screen list
		Button EditChapScreenButton = new Button("Edit Chapter Screens");
		EditChapScreenButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				StringVecEditor SVE =
					new StringVecEditor(Parent,SD.ChapScreens,SD.FileName + " Chapter Screens");
			}}
		);
		C.gridy = (Row++);
		add(EditChapScreenButton,C);
		
		// Edit the FMV list
		Button EditFMVButton = new Button("Edit FMV's");
		EditFMVButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				StringVecEditor SVE =
					new StringVecEditor(Parent,SD.FMVs,SD.FileName + " FMV's");
			}}
		);
		C.gridy = (Row++);
		add(EditFMVButton,C);
		
		// Edit the cutscene list
		Button EditCutsceneButton = new Button("Edit Cutscenes");
		EditCutsceneButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				StringVecEditor SVE =
					new StringVecEditor(Parent,SD.Cutscenes,SD.FileName + " Cutscenes");
			}}
		);
		C.gridy = (Row++);
		add(EditCutsceneButton,C);
		
		// Edit the initial script
		Button EditStartScriptButton = new Button("Edit Startup Script");
		EditStartScriptButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				LevelScriptEditor LSE =
					new LevelScriptEditor(Parent,SD.StartupScript,SD.FileName + " Startup Script");
				LSE.dispose();
			}}
		);
		C.gridy = (Row++);
		add(EditStartScriptButton,C);
		
		// Edit game strings set 1
		Button EditGS1Button = new Button("Edit Game Strings 1");
		EditGS1Button.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				StringListEditor SLE =
					new StringListEditor(Parent,SD.GameStrings1,SD.FileName + " Game Strings 1");
			}}
		);
		C.gridy = (Row++);
		add(EditGS1Button,C);
		
		// Edit game strings set 2
		Button EditGS2Button = new Button("Edit Game Strings 2");
		EditGS2Button.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				StringListEditor SLE =
					new StringListEditor(Parent,SD.GameStrings2,SD.FileName + " Game Strings 2");
			}}
		);
		C.gridy = (Row++);
		add(EditGS2Button,C);
		
		// Put in some spacing
		Row++;
		
		// Save this file
		Button SaveButton = new Button("Save");
		SaveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				// Update stuff to be saved
				UpdateScriptData();
				
				FileDialog FD = new FileDialog(Parent,"Save TOMBPC.DAT file",FileDialog.SAVE);
				FD.setDirectory(DirName);
				FD.setFile(FileName);
				FD.setVisible(true);
				DirName = FD.getDirectory();
				FileName = FD.getFile();
				if (FileName == null) return;
				File OutFile = new File(DirName,FileName);
		
				OutputStream OS;
				
				try {
					OS = (OutputStream)new FileOutputStream(OutFile);
					if (!ScriptIO.Write(Parent, OS, SD))
						throw new Exception("Write failed");
					OS.close();
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Save Error for " + FileName, 480,48, X.toString());
					return;
				}
			}}
		);
		C.gridy = (Row++);
		add(SaveButton,C);
		
		// Quit editing this file
		Button CloseButton = new Button("Close");
		CloseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {CloseAction();}}
		);
		C.gridy = (Row++);
		add(CloseButton,C);
		validate();
		setVisible(true);
		
		// Handle the close box:
		addWindowListener(new WindowListener()
			{
			public void windowOpened(WindowEvent E) {}
			public void windowClosing(WindowEvent E) {CloseAction();}
			public void windowClosed(WindowEvent E) {}
			public void windowIconified(WindowEvent E) {}
			public void windowDeiconified(WindowEvent E) {}
			public void windowActivated(WindowEvent E) {}
			public void windowDeactivated(WindowEvent E) {}
			}
		);
		
		setSize(getPreferredSize());
		validate();
	}
}
