/*

	These routines do the script-data I/O
	
	I'm implementing them as static function objects

*/

package tr_script;

import java.awt.*;
import java.io.*;
import java.util.*;

import util.*;


// Hack to produce an array of variable-length arrays
class ByteCtnr {
	byte[] ByteList;
}
class ShortCtnr {
	short[] ShortList;
}
class StringCtnr {
	String[] StringList;
}


public final class ScriptIO {
	
	private static ByteCtnr[] ReadByteSet(InputStream IS, int Num)
		throws IOException {
		
		// Read in offset table:
		byte[] ShortBuf = new byte[2];
		short[] OffsetTable = new short[Num+1];
		for (int n=0; n<=Num; n++) {
			IS.read(ShortBuf);
			OffsetTable[n] = Converter.GetLittleEndianShort(ShortBuf,0);
		}
		
		// Get the byte strings:
		ByteCtnr[] ByteSet = new ByteCtnr[Num];
		byte[] Buf = new byte[OffsetTable[Num]];
		IS.read(Buf);
		for (int n=0; n<Num; n++) {
			ByteSet[n] = new ByteCtnr();
			ByteSet[n].ByteList =
				Converter.GetBytes(Buf, OffsetTable[n], OffsetTable[n+1] - OffsetTable[n]);
		}
		
		return ByteSet;
	}
	
	private static void WriteByteSet(OutputStream OS, ByteCtnr[] ByteSet)
		throws IOException {
		
		// Write out offset table:
		int Num = ByteSet.length;
		byte[] ShortBuf = new byte[2];		
		short Offset = 0;
		Converter.PutLittleEndianShort(ShortBuf,0,Offset);
		OS.write(ShortBuf);
		for (int n=0; n<Num; n++) {
			Offset += ByteSet[n].ByteList.length;
			Converter.PutLittleEndianShort(ShortBuf,0,Offset);
			OS.write(ShortBuf);
		}
		
		// Write out the bytes themselves:
		for (int n=0; n<Num; n++)
			OS.write(ByteSet[n].ByteList);
	}
	
	private static ShortCtnr[] ReadShortSet(InputStream IS, int Num)
		throws IOException {
	
		// Read in bytes to convert:
		ByteCtnr[] ByteSet = ReadByteSet(IS,Num);
		
		// Do the conversion:
		ShortCtnr[] ShortSet = new ShortCtnr[Num];
		for (int n=0; n<Num; n++) {
			byte[] BBuf = ByteSet[n].ByteList;
			int nshort = BBuf.length/2;
			short[] SBuf = new short[nshort];
			for (int k=0; k<nshort; k++)
				SBuf[k] = Converter.GetLittleEndianShort(BBuf,2*k);
			ShortSet[n] = new ShortCtnr();
			ShortSet[n].ShortList = SBuf;
		}
		
		return ShortSet;
	}
	
	private static void WriteShortSet(OutputStream OS, ShortCtnr[] ShortSet)
		throws IOException {
	
		// Set up output byte array:
		int Num = ShortSet.length;
		ByteCtnr[] ByteSet = new ByteCtnr[Num];
		
		// Do the conversion:
		for (int n=0; n<Num; n++) {
			short[] SBuf = ShortSet[n].ShortList;
			int nshort = SBuf.length;
			byte[] BBuf = new byte[2*nshort];
			
			for (int k=0; k<nshort; k++)
				Converter.PutLittleEndianShort(BBuf,2*k,SBuf[k]);
			
			ByteSet[n] = new ByteCtnr();
			ByteSet[n].ByteList = BBuf;
		}
		
		WriteByteSet(OS, ByteSet);
	}	
	
	private static String[] ReadStringSet(InputStream IS, int Num, byte XORByte)
		throws IOException {
	
		// Read in bytes to convert:
		ByteCtnr[] ByteSet = ReadByteSet(IS,Num);
		
		// Do the conversion:
		String[] StringSet = new String[Num];
		for (int n=0; n<Num; n++) {
			byte[] BBuf = (byte[])ByteSet[n].ByteList;
			
			// Do the XOR
			if (XORByte != 0) {
				for (int b=0; b<BBuf.length; b++)
					BBuf[b] ^= XORByte;
			}

			StringSet[n] = Converter.GetCharsDelimited(BBuf,0,(byte)0);
		}
		
		return StringSet;
	}
	
	private static void WriteStringSet(OutputStream OS, String[] StringSet, byte XORByte)
		throws IOException {
		
		// Set up output byte array:
		int Num = StringSet.length;
		ByteCtnr[] ByteSet = new ByteCtnr[Num];
		
		// Do the conversion:
		for (int n=0; n<Num; n++) {

			// Create zero-padded list (C string)
			byte[] NewBytes = StringSet[n].getBytes();
			
			int len = NewBytes.length;
			byte[] BBuf = new byte[len+1];
			for (int b=0; b<len; b++)
				BBuf[b] = NewBytes[b];
			BBuf[len] = (byte)0;
			
			// Do the XOR
			if (XORByte != 0) {
				for (int b=0; b<BBuf.length; b++)
					BBuf[b] ^= XORByte;
			}
			
			ByteSet[n] = new ByteCtnr();
			ByteSet[n].ByteList = BBuf;
		}
		
		WriteByteSet(OS, ByteSet);
	}
	
	private static Vector StringList2Vec(String[] StringList) {
		Vector Vec = new Vector(StringList.length);
		for (int ic=0; ic<StringList.length; ic++)
			Vec.addElement(StringList[ic]);
			
		return Vec;
	}
	
	private static String[] Vec2StringList(Vector Vec) {
		String[] StringList = new String[Vec.size()];
		for (int ic=0; ic<Vec.size(); ic++)
			StringList[ic] = (String)Vec.elementAt(ic);
			
		return StringList;
	}
	
	private static Vector ShortList2Vec(short[] ShortList) {
		Vector Vec = new Vector(ShortList.length);
		for (int ic=0; ic<ShortList.length; ic++)
			Vec.addElement(new Short(ShortList[ic]));
			
		return Vec;
	}
	
	private static short[] Vec2ShortList(Vector Vec) {
		short[] ShortList = new short[Vec.size()];
		for (int ic=0; ic<Vec.size(); ic++)
			ShortList[ic] = ((Short)Vec.elementAt(ic)).shortValue();
			
		return ShortList;
	}
	
	private static Vector LevelList2Vec(LevelData[] LevelList) {
		Vector LevelVec = new Vector(LevelList.length);
		for (int ic=0; ic<LevelList.length; ic++)
			LevelVec.addElement(LevelList[ic]);
			
		return LevelVec;
	}
	
	private static LevelData[] Vec2LevelList(Vector LevelVec) {
		LevelData[] LevelList = new LevelData[LevelVec.size()];
		for (int ic=0; ic<LevelVec.size(); ic++)
			LevelList[ic] = (LevelData)LevelVec.elementAt(ic);
		
		return LevelList;
	}
	
	// Read a script: return an empty object if there was a read error
	public static boolean Read(Frame Parent, InputStream IS, ScriptData SD) {
		
		// Buffers:
		byte ShortBuf[] = new byte[2];
		byte IntBuf[] = new byte[4];
		
		// Watch for I/O errors:
		try {
			IS.read(IntBuf);
			SD.Version = Converter.GetLittleEndianInt(IntBuf,0);
			
			if (SD.Version != ScriptData.VersionValue) {
				Integer VR = new Integer(SD.Version);
				Integer VT = new Integer(ScriptData.VersionValue);
				String Message = "Version number " + VR + " is not " + VT;
				throw new Exception(Message);
			}
			
			byte[] InfoBuf = new byte[ScriptData.InfoLen];
			IS.read(InfoBuf);
			SD.Info = Converter.GetCharsDelimited(InfoBuf,0,(byte)0);
			
			IS.read(ShortBuf);
			int HeaderLen = Converter.GetLittleEndianShort(ShortBuf,0);
			if (HeaderLen != ScriptData.HeaderLen) {
				Integer HR = new Integer(HeaderLen);
				Integer HT = new Integer(ScriptData.HeaderLen);
				String Message = "Header size " + HR + " is not " + HT;
				throw new Exception(Message);
			}
			
			byte HeaderBuf[] = new byte[ScriptData.HeaderLen];
			IS.read(HeaderBuf);
			
			int Loc = 0;
			
			SD.FirstOption = Converter.GetLittleEndianInt(HeaderBuf,Loc);
			Loc += 4;
			
			SD.TitleReplace = Converter.GetLittleEndianInt(HeaderBuf,Loc);
			Loc += 4;
			
			SD.OnDeathDemoMode = Converter.GetLittleEndianInt(HeaderBuf,Loc);
			Loc += 4;
			
			SD.OnDeathInGame = Converter.GetLittleEndianInt(HeaderBuf,Loc);
			Loc += 4;
			
			SD.DemoTime = Converter.GetLittleEndianInt(HeaderBuf,Loc);
			Loc += 4;
			
			SD.OnDemoInterrupt = Converter.GetLittleEndianInt(HeaderBuf,Loc);
			Loc += 4;
			
			SD.OnDemoEnd = Converter.GetLittleEndianInt(HeaderBuf,Loc);
			Loc += 4;
	    	
	    	SD.Unused1 = Converter.GetBytes(HeaderBuf,Loc,ScriptData.Unused1Len);
	    	Loc += ScriptData.Unused1Len;
			
			SD.NumLevels = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
			SD.NumChapScreens = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
			SD.NumTitles = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
			SD.NumFMVs = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
			SD.NumCutscenes = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
			SD.NumDemoLevels = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
			SD.TitleSoundID = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
			SD.SingleLevel = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
	    	SD.Unused2 = Converter.GetBytes(HeaderBuf,Loc,ScriptData.Unused2Len);
	    	Loc += ScriptData.Unused2Len;
	    	
			SD.Flags = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
	    	SD.Unused3 = Converter.GetBytes(HeaderBuf,Loc,ScriptData.Unused3Len);
	    	Loc += ScriptData.Unused3Len;
			
			SD.XORByte = HeaderBuf[Loc];
			Loc++;
			
			SD.BowdlerByte = HeaderBuf[Loc];
			Loc++;
 			
			SD.SecretSoundID = Converter.GetLittleEndianShort(HeaderBuf,Loc);
			Loc += 2;
			
	    	SD.Unused5 = Converter.GetBytes(HeaderBuf,Loc,ScriptData.Unused5Len);
	    	Loc += ScriptData.Unused5Len;
			
			if (Loc != ScriptData.HeaderLen) {
				Integer HR = new Integer(Loc);
				Integer HT = new Integer(ScriptData.HeaderLen);
				String Message = "Header out of sync: " + HR + " is not " + HT;
				throw new Exception(Message);
			}
			
			// Actual XOR byte to use
			byte XORByte = ((SD.Flags & ScriptData.Flag_Use_Encryption) != 0) ? SD.XORByte : 0;
			
			String[] LevelNames = ReadStringSet(IS,SD.NumLevels,XORByte);

			SD.ChapScreens = StringList2Vec(ReadStringSet(IS,SD.NumChapScreens,XORByte));
			
			SD.Titles = StringList2Vec(ReadStringSet(IS,SD.NumTitles,XORByte));

			SD.FMVs = StringList2Vec(ReadStringSet(IS,SD.NumFMVs,XORByte));
			
			String[] LevelFiles = ReadStringSet(IS,SD.NumLevels,XORByte);
			
			SD.Cutscenes = StringList2Vec(ReadStringSet(IS,SD.NumCutscenes,XORByte));
			
			ShortCtnr[] LevelScripts = ReadShortSet(IS,SD.NumLevels+1);
			
			if (SD.NumDemoLevels > 0) {
				SD.DemoLevels = new short[SD.NumDemoLevels];
				for (int n=0; n<SD.NumDemoLevels; n++) {
					IS.read(ShortBuf);
					SD.DemoLevels[n] = Converter.GetLittleEndianShort(ShortBuf,0);
				}
			}
			
			if (SD.IsPSX) {
				int NumExtraPSXData = 2*SD.NumFMVs;
				SD.ExtraPSXData = new int[NumExtraPSXData];
				for (int n=0; n<NumExtraPSXData; n++) {
					IS.read(IntBuf);
					SD.ExtraPSXData[n] = Converter.GetLittleEndianInt(IntBuf,0);
				}
			}
			
			IS.read(ShortBuf);
			SD.NumGameStrings1 = Converter.GetLittleEndianShort(ShortBuf,0);
			
			SD.GameStrings1 = ReadStringSet(IS,SD.NumGameStrings1,XORByte);
			
			SD.GameStrings2 = ReadStringSet(IS,SD.NumGameStrings2,XORByte);
			
			StringCtnr[] KeyStrings = new StringCtnr[LevelData.NumKeys];
			for (int k=0; k<LevelData.NumKeys; k++) {
				StringCtnr KeySet = new StringCtnr();
				KeySet.StringList = ReadStringSet(IS,SD.NumLevels,XORByte);
				KeyStrings[k] = KeySet;
			}
			
			// Rearrange the per-level data into a form more suitable for editing
			
			SD.StartupScript = ShortList2Vec(LevelScripts[0].ShortList);
			
			LevelData[] LevelList = new LevelData[SD.NumLevels];
			
			for (int n=0; n<SD.NumLevels; n++) {
				LevelData Level = new LevelData();
				
				Level.Name = LevelNames[n];
				Level.FileName = LevelFiles[n];
				Level.IfDemo = false;
				Level.Script = ShortList2Vec(LevelScripts[n+1].ShortList);
				for (int k=0; k<LevelData.NumKeys; k++) {
					Level.KeyNames[k] = KeyStrings[k].StringList[n];
				}
				
				LevelList[n] = Level;
			}
			
			for (int n=0; n<SD.NumDemoLevels; n++) {
				LevelList[SD.DemoLevels[n]].IfDemo = true;
			}
			
			SD.Levels = LevelList2Vec(LevelList);
			
		} catch (Exception X) {
			InfoAlert A =
				new InfoAlert(Parent, "Read Error for " + SD.FileName, 480,72, X.toString());
			
			return false;
		}
		
		return true;
	}
	
	// Write a script
	public static boolean Write(Frame Parent, OutputStream OS, ScriptData SD) {
		
		// Keep in sync
		SD.Synchronize();
		
		// Buffers:
		byte ShortBuf[] = new byte[2];
		byte IntBuf[] = new byte[4];
		
		// Watch for I/O errors:
		try {
			Converter.PutLittleEndianInt(IntBuf,0,SD.Version);
			OS.write(IntBuf);
			
			byte[] InfoBuf = new byte[ScriptData.InfoLen];
			Converter.PutCharsFixed(InfoBuf,0,ScriptData.InfoLen,SD.Info);
			OS.write(InfoBuf);
			
			Converter.PutLittleEndianShort(ShortBuf,0,(short)ScriptData.HeaderLen);
			OS.write(ShortBuf);

			byte HeaderBuf[] = new byte[ScriptData.HeaderLen];
			
			int Loc = 0;
			
			Converter.PutLittleEndianInt(HeaderBuf,Loc,SD.FirstOption);
			Loc += 4;
			
			Converter.PutLittleEndianInt(HeaderBuf,Loc,SD.TitleReplace);
			Loc += 4;
			
			Converter.PutLittleEndianInt(HeaderBuf,Loc,SD.OnDeathDemoMode);
			Loc += 4;
			
			Converter.PutLittleEndianInt(HeaderBuf,Loc,SD.OnDeathInGame);
			Loc += 4;
			
			Converter.PutLittleEndianInt(HeaderBuf,Loc,SD.DemoTime);
			Loc += 4;
			
			Converter.PutLittleEndianInt(HeaderBuf,Loc,SD.OnDemoInterrupt);
			Loc += 4;
			
			Converter.PutLittleEndianInt(HeaderBuf,Loc,SD.OnDemoEnd);
			Loc += 4;
			
			Converter.PutBytes(HeaderBuf,Loc,SD.Unused1);
			Loc += ScriptData.Unused1Len;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.NumLevels);
			Loc += 2;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.NumChapScreens);
			Loc += 2;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.NumTitles);
			Loc += 2;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.NumFMVs);
			Loc += 2;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.NumCutscenes);
			Loc += 2;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.NumDemoLevels);
			Loc += 2;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.TitleSoundID);
			Loc += 2;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.SingleLevel);
			Loc += 2;
			
			Converter.PutBytes(HeaderBuf,Loc,SD.Unused2);
			Loc += ScriptData.Unused2Len;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.Flags);
			Loc += 2;
			
			Converter.PutBytes(HeaderBuf,Loc,SD.Unused3);
			Loc += ScriptData.Unused3Len;
			
			HeaderBuf[Loc] = SD.XORByte;
			Loc++;
			
			HeaderBuf[Loc] = SD.BowdlerByte;
			Loc++;
			
			Converter.PutLittleEndianShort(HeaderBuf,Loc,SD.SecretSoundID);
			Loc += 2;
			
			Converter.PutBytes(HeaderBuf,Loc,SD.Unused5);
			Loc += ScriptData.Unused5Len;
			
			OS.write(HeaderBuf);
			
			if (Loc != ScriptData.HeaderLen) {
				Integer HR = new Integer(Loc);
				Integer HT = new Integer(ScriptData.HeaderLen);
				String Message = "Header out of sync: " + HR + " is not " + HT;
				throw new Exception(Message);
			}
			
			// Actual XOR byte to use
			byte XORByte = ((SD.Flags & ScriptData.Flag_Use_Encryption) != 0) ? SD.XORByte : 0;
			
			// Rearrange the per-level data into a form more suitable for writing out
			
			LevelData[] LevelList = Vec2LevelList(SD.Levels);
			
			String[] LevelNames = new String[SD.NumLevels];
			
			String[] LevelFiles = new String[SD.NumLevels];
			
			ShortCtnr[] LevelScripts = new ShortCtnr[SD.NumLevels+1];
			for (int n=0; n<=SD.NumLevels; n++)
				LevelScripts[n] = new ShortCtnr();
			
			LevelScripts[0].ShortList = Vec2ShortList(SD.StartupScript);
			
			StringCtnr[] KeyStrings = new StringCtnr[LevelData.NumKeys];
			for (int n=0; n<LevelData.NumKeys; n++) {
				StringCtnr KeySet = new StringCtnr();
				KeySet.StringList = new String[SD.NumLevels];
				KeyStrings[n] = KeySet;
			}
			
			for (int n=0; n<SD.NumLevels; n++) {
				LevelData Level = LevelList[n];
				LevelNames[n] = Level.Name;
				LevelFiles[n] = Level.FileName;
				LevelScripts[n+1].ShortList = Vec2ShortList(Level.Script);
				for (int k=0; k<LevelData.NumKeys; k++) {
					KeyStrings[k].StringList[n] = Level.KeyNames[k];
				}
			}
			
			WriteStringSet(OS,LevelNames,XORByte);
			
			WriteStringSet(OS,Vec2StringList(SD.ChapScreens),XORByte);
			
			WriteStringSet(OS,Vec2StringList(SD.Titles),XORByte);
			
			WriteStringSet(OS,Vec2StringList(SD.FMVs),XORByte);
			
			WriteStringSet(OS,LevelFiles,XORByte);
			
			WriteStringSet(OS,Vec2StringList(SD.Cutscenes),XORByte);
			
			WriteShortSet(OS,LevelScripts);
			
			for (int n=0; n<SD.NumDemoLevels; n++) {
				Converter.PutLittleEndianShort(ShortBuf,0,SD.DemoLevels[n]);
				OS.write(ShortBuf);
			}
			
			if (SD.IsPSX) {
				// Put in dummy data if the number of FMV's has changed
				int NumExtraPSXData = 2*SD.NumFMVs;
				if (NumExtraPSXData == SD.ExtraPSXData.length) {
					for (int n=0; n<NumExtraPSXData; n++) {
						Converter.PutLittleEndianInt(IntBuf,0,SD.ExtraPSXData[n]);
						OS.write(IntBuf);
					}
				} else {
					for (int n=0; n<NumExtraPSXData; n++) {
						int Fake = ((n % 2) == 1) ? 0 : 1;
						Converter.PutLittleEndianInt(IntBuf,0,Fake);
						OS.write(IntBuf);
					}
				}
			}
			
			Converter.PutLittleEndianShort(ShortBuf,0,SD.NumGameStrings1);
			OS.write(ShortBuf);
			
			WriteStringSet(OS,SD.GameStrings1,XORByte);
			
			WriteStringSet(OS,SD.GameStrings2,XORByte);
			
			for (int k=0; k<LevelData.NumKeys; k++)
				WriteStringSet(OS,KeyStrings[k].StringList,XORByte);
			
		} catch (Exception X) {
			InfoAlert A =
				new InfoAlert(Parent, "Write Error for " + SD.FileName, 480,72, X.toString());
			return false;
		}
		
		return true;
	}
}
