// String-list editor: use for game strings (always a fixed number of them)

package tr_script;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import util.*;


final class StringListEditor extends Dialog {
	
	// Stuff that has to be local
	private Frame Parent;
	private String[] StringList;
		
	// Displayed list of strings:	
	private List DispList;
	
	// Field for editing string:
	private TextField EditField;
	
	// Create a string in a suitable format for the display list:
	private String MakeDispString(int ic, String ListStr) {
		return (Integer.toString(ic) + ": " + ListStr);
	}
		
	StringListEditor(Frame _Parent, String[] _StringList, String Name) {
		super(_Parent,Name);
		setSize(384,256);
		setResizable(false);
		
		// Necessary for some of the inner classes
		Parent = _Parent;
		StringList = _StringList;
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
		
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// The display list itself
		DispList = new List(16);
		for (int ic=0; ic<StringList.length; ic++)
			DispList.add(MakeDispString(ic,StringList[ic]));
		
		// Action is to get a string from the list
		DispList.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				EditField.setText(StringList[ic]);
			}}
		);
		
		C.gridx = 0;
		C.gridheight = 16;
		C.anchor = GridBagConstraints.NORTHWEST;
		add(DispList,C);
		C.gridheight = 1;
		
		// The edit-text field
		EditField = new TextField(24);
		int Row = 0;
		C.gridx = 1;
		C.gridy = (Row++);
		C.anchor = GridBagConstraints.WEST;
		add(EditField,C);
		
		// The control buttons
		C.fill = GridBagConstraints.HORIZONTAL;
		C.anchor = GridBagConstraints.WEST;
		
		// Get the currently-selected string
		Button GetButton = new Button("Get from List");
		GetButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				EditField.setText(StringList[ic]);
			}}
		);
		C.gridy = (Row++);
		add(GetButton,C);
		
		// Set the currently-selected string to the text-field contents
		Button ChangeButton = new Button("Change in List");
		ChangeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				String NewString =  EditField.getText();
				StringList[ic] = NewString;
				DispList.replaceItem(MakeDispString(ic,NewString),ic);
					
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(ChangeButton,C);
		
		// Load the strings from a file
		Button LoadButton = new Button("Load Strings");
		LoadButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Load list from file",FileDialog.LOAD);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File InFile = new File(DirName,FileName);
				
				FileReader FR;
				
				try {
					FR = new FileReader(InFile);
					
					// So I don't have to do this by hand (heh heh)
					StreamTokenizer Grabber = new StreamTokenizer(FR);
					Grabber.resetSyntax();
					Grabber.wordChars(0,255);
					Grabber.whitespaceChars('\n','\n');
					Grabber.whitespaceChars('\r','\r');
					
					int ic = 0;
					while(ic < StringList.length) {
						int TokenType = Grabber.nextToken();
						if (TokenType == StreamTokenizer.TT_WORD) {
							String ReadString = Grabber.sval;
							if (ReadString.length() != 0)
								StringList[ic++] = ReadString;
						} else if (TokenType == StreamTokenizer.TT_EOF) break;
					}
					FR.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Load Error for " + FileName, 480,48, X.toString());
				} finally {
					for (int ic=0; ic<StringList.length; ic++)
						DispList.replaceItem(MakeDispString(ic,StringList[ic]),ic);
					
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(LoadButton,C);
		
		// Save the strings to a file
		Button SaveButton = new Button("Save Strings");
		SaveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Save list in file",FileDialog.SAVE);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File OutFile = new File(DirName,FileName);
				
				FileWriter FW;
				
				try {
					FW = new FileWriter(OutFile);
				
					for (int ic=0; ic<StringList.length; ic++) {
						FW.write(StringList[ic]);
						FW.write(System.getProperty("line.separator"));
					}
					FW.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Save Error for " + FileName, 480,48, X.toString());
					return;
				}
			}}
		);
		C.gridy = (Row++);
		add(SaveButton,C);
		
		// Close this dialog box (changes are saved automatically)
		Button CloseButton = new Button("Close");
		CloseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {dispose();}}
		);
		C.gridy = (Row++);
		add(CloseButton,C);

		// Handle the close box:
		addWindowListener(new WindowListener()
			{
			public void windowOpened(WindowEvent E) {}
			public void windowClosing(WindowEvent E) {dispose();}
			public void windowClosed(WindowEvent E) {}
			public void windowIconified(WindowEvent E) {}
			public void windowDeiconified(WindowEvent E) {}
			public void windowActivated(WindowEvent E) {}
			public void windowDeactivated(WindowEvent E) {}
			}
		);
		
		validate();
		setVisible(true);
		
		setSize(getPreferredSize());
		validate();
	}
}
