// String-vector editor: use for lists of files (variable number of them)

package tr_script;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import util.*;


final class StringVecEditor extends Dialog {
	
	// Stuff that has to be local
	private Frame Parent;
	private Vector StringVec;
	
	// Displayed list of strings:
	private List DispList;
	
	// Field for editing string:
	private TextField EditField;
	
	// Create a string in a suitable format for the display list:
	private String MakeDispString(int ic, String ListStr) {
		return (Integer.toString(ic) + ": " + ListStr);
	}
	
	// Rebuild the display list
	private void MakeDispList() {
		DispList.removeAll();
		for (int ic=0; ic<StringVec.size(); ic++)
			DispList.add(MakeDispString(ic,(String)StringVec.elementAt(ic)));
	}
		
	StringVecEditor(Frame _Parent, Vector _StringVec, String Name) {
		super(_Parent,Name);
		setSize(384,256);
		setResizable(false);
		
		// Necessary for some of the inner classes
		Parent = _Parent;
		StringVec = _StringVec;
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
		
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// The display list itself
		DispList = new List(16);
		MakeDispList();
		
		// Action is to get a string from the list
		DispList.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				EditField.setText((String)StringVec.elementAt(ic));
			}}
		);
		
		C.gridx = 0;
		C.gridheight = 16;
		C.anchor = GridBagConstraints.NORTHWEST;
		add(DispList,C);
		C.gridheight = 1;
		
		// The edit-text field
		EditField = new TextField(24);
		int Row = 0;
		C.gridx = 1;
		C.gridy = (Row++);
		C.anchor = GridBagConstraints.WEST;
		add(EditField,C);
		
		// The control buttons
		C.fill = GridBagConstraints.HORIZONTAL;
		C.anchor = GridBagConstraints.WEST;
		
		// Get the currently-selected string
		Button GetButton = new Button("Get from List");
		GetButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				EditField.setText((String)StringVec.elementAt(ic));
			}}
		);
		C.gridy = (Row++);
		add(GetButton,C);
		
		// Set the currently-selected string to the text-field contents
		Button ChangeButton = new Button("Change in List");
		ChangeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				String NewString =  EditField.getText();
				StringVec.setElementAt(NewString,ic);
				DispList.replaceItem(MakeDispString(ic,NewString),ic);
					
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(ChangeButton,C);

		// Duplicate the current string
		Button DuplicateButton = new Button("Duplicate");
		DuplicateButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				String Str = new String((String)StringVec.elementAt(ic));
				StringVec.insertElementAt(Str,ic);
				MakeDispList();
				DispList.select(ic);
			}}
		);
		C.gridy = (Row++);
		add(DuplicateButton,C);

		// Move the current string to some location
		Button MoveButton = new Button("Move");
		MoveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				int SVLen = StringVec.size();
				String[] LocList = new String[SVLen + 1];
				for (int ix=0; ix<SVLen; ix++) {
					LocList[ix] = (String)StringVec.elementAt(ix);
				}
				LocList[SVLen] = "<<< End >>>";
				String Str = (String)StringVec.elementAt(ic);
				StringLocSelector SLE = 
					new StringLocSelector(Parent,"String-List Editor",
					"Move string \""+Str+"\" to before",LocList);
				if (SLE.DoGoto) {
					// This bumps down all strings after it
					StringVec.removeElementAt(ic);
					int Loc = SLE.Loc;
					if (Loc > ic) Loc--;
					StringVec.insertElementAt(Str,Loc);
					MakeDispList();
					DispList.select(Loc);
				}
			}}
		);
		C.gridy = (Row++);
		add(MoveButton,C);

		// Insert a string before the currently-selected string location
		Button InsertBeforeButton = new Button("Insert Before");
		InsertBeforeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (StringVec.size() <= 0) {
					StringVec.addElement(EditField.getText());
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					StringVec.insertElementAt(EditField.getText(),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertBeforeButton,C);
		
		// Insert a string after the currently-selected string location
		Button InsertAfterButton = new Button("Insert After");
		InsertAfterButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (StringVec.size() <= 0) {
					StringVec.addElement(EditField.getText());
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					ic++;
					StringVec.insertElementAt(EditField.getText(),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertAfterButton,C);

		// Delete the currently-selected string
		Button DeleteButton = new Button("Delete");
		DeleteButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				WarningAlert A = new WarningAlert(Parent,"Warning!",384,72,
					"Really want to delete \"" + (String)StringVec.elementAt(ic) + "\"?");
				if (A.If_OK) {
					StringVec.removeElementAt(ic);
					MakeDispList();
					DispList.select(ic);
				
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(DeleteButton,C);
		
		// Load the strings from a file
		Button LoadButton = new Button("Load Strings");
		LoadButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Load list from file",FileDialog.LOAD);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File InFile = new File(DirName,FileName);
				
				FileReader FR;
				
				StringVec.removeAllElements();
				
				try {
					FR = new FileReader(InFile);
					
					// So I don't have to do this by hand (heh heh)
					StreamTokenizer Grabber = new StreamTokenizer(FR);
					Grabber.resetSyntax();
					Grabber.wordChars(0,255);
					Grabber.whitespaceChars('\n','\n');
					Grabber.whitespaceChars('\r','\r');
					
					while(true) {
						int TokenType = Grabber.nextToken();
						if (TokenType == StreamTokenizer.TT_WORD) {
							String ReadString = Grabber.sval;
							if (ReadString.length() != 0)
								StringVec.addElement(ReadString);
						} else if (TokenType == StreamTokenizer.TT_EOF) break;
					}
					FR.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Load Error for " + FileName, 480,48, X.toString());
				} finally {
					MakeDispList();
					
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(LoadButton,C);
		
		// Save the strings to a file
		Button SaveButton = new Button("Save Strings");
		SaveButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				FileDialog FD = new FileDialog(Parent,"Save list in file",FileDialog.SAVE);
				FD.setVisible(true);
				String DirName = FD.getDirectory();
				String FileName = FD.getFile();
				if (FileName == null) return;
				File OutFile = new File(DirName,FileName);
				
				FileWriter FW;
				
				try {
					FW = new FileWriter(OutFile);
				
					for (int ic=0; ic<StringVec.size(); ic++) {
						FW.write((String)StringVec.elementAt(ic));
						FW.write(System.getProperty("line.separator"));
					}
					FW.close();
					
				} catch (Exception X) {
					InfoAlert A =
						new InfoAlert(Parent, "Save Error for " + FileName, 480,48, X.toString());
					return;
				}
			}}
		);
		C.gridy = (Row++);
		add(SaveButton,C);
				
		// Close this dialog box (changes are saved automatically)
		Button CloseButton = new Button("Close");
		CloseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {dispose();}}
		);
		C.gridy = (Row++);
		add(CloseButton,C);

		// Handle the close box:
		addWindowListener(new WindowListener()
			{
			public void windowOpened(WindowEvent E) {}
			public void windowClosing(WindowEvent E) {dispose();}
			public void windowClosed(WindowEvent E) {}
			public void windowIconified(WindowEvent E) {}
			public void windowDeiconified(WindowEvent E) {}
			public void windowActivated(WindowEvent E) {}
			public void windowDeactivated(WindowEvent E) {}
			}
		);
		
		validate();
		setVisible(true);
		
		setSize(getPreferredSize());
		validate();
	}
}

