/*

	This class is the master-dialog class
	
*/

package tr_unpack;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.zip.*;

import util.*;


public final class MasterDialog extends Frame {
	
	//  Make the open directory and filename persistent
	private String DirName, FileName;
	
	// Unpack-file routine
	private void UnpackFile() {
		FileDialog FD = new FileDialog(this,"What PAK file to unpack?",FileDialog.LOAD);
		FD.setDirectory(DirName);
		FD.setFile(FileName);
		FD.setVisible(true);
		String DirName = FD.getDirectory();
		String FileName = FD.getFile();
		if (FileName == null) return;
		File InFile = new File(DirName,FileName);
		
		InputStream IS;
		
		try {
			IS = (InputStream)new FileInputStream(InFile);
			
			// Get the unpacked size
			byte IntBuf[] = new byte[4];
			IS.read(IntBuf);
			int UnpackedSize = Converter.GetLittleEndianInt(IntBuf,0);
			
			InfoAlert A =
				new InfoAlert(this, "Info for " + FileName, 240,48, "Unpacked size is " + UnpackedSize);
			
			// Create unpacked file
			
			FD = new FileDialog(Parent,"Save unpacked \"" + FileName + "\" as",FileDialog.SAVE);
			FD.setDirectory(DirName);
			FD.setFile(FileName + ".dat");
			FD.setVisible(true);
			DirName = FD.getDirectory();
			FileName = FD.getFile();
			if (FileName == null) return;
			File OutFile = new File(DirName,FileName);
			
			OutputStream OS = (OutputStream)new FileOutputStream(OutFile);
			
			// Unpack the file
			InflaterInputStream IIS = new InflaterInputStream(IS);
			
			int BufferSize = 4096;
			byte[] Buffer = new byte[BufferSize];

			int HowMuchLeft = UnpackedSize;			
			while(HowMuchLeft > 0) {
				int NumToRead = HowMuchLeft > BufferSize ? BufferSize : HowMuchLeft;
				int ActualNumberRead = IIS.read(Buffer,0,NumToRead);
				if (ActualNumberRead == -1) throw new Exception("Premature end-of-file reached");
				OS.write(Buffer,0,ActualNumberRead);
				HowMuchLeft -= ActualNumberRead;
			}
			
			IS.close();
			OS.close();
		} catch (Exception X) {
			InfoAlert A =
				new InfoAlert(this, "Processing Error for " + FileName, 480,48, X.toString());
			return;
		}
	}
	
	// Pack-file routine
	private void PackFile() {
		FileDialog FD = new FileDialog(this,"What file to pack?",FileDialog.LOAD);
		FD.setDirectory(DirName);
		FD.setFile(FileName);
		FD.setVisible(true);
		String DirName = FD.getDirectory();
		String FileName = FD.getFile();
		if (FileName == null) return;
		File InFile = new File(DirName,FileName);
		
		InputStream IS;
		
		try {
			IS = (InputStream)new FileInputStream(InFile);
			
			// Get the unpacked size
			int UnpackedSize = (int)InFile.length();
			
			InfoAlert A =
				new InfoAlert(this, "Info for " + FileName, 240,48, "Unpacked size is " + UnpackedSize);
			
			// Create packed file
			FD = new FileDialog(Parent,"Save packed \"" + FileName + "\" as",FileDialog.SAVE);
			FD.setDirectory(DirName);
			FD.setFile(FileName + ".pak");
			FD.setVisible(true);
			DirName = FD.getDirectory();
			FileName = FD.getFile();
			if (FileName == null) return;
			File OutFile = new File(DirName,FileName);
			
			OutputStream OS = (OutputStream)new FileOutputStream(OutFile);
			
			// Pack the file
			byte IntBuf[] = new byte[4];
			Converter.PutLittleEndianInt(IntBuf,0,UnpackedSize);
			OS.write(IntBuf);
			
			DeflaterOutputStream DOS = new DeflaterOutputStream(OS);
			
			int BufferSize = 4096;
			byte[] Buffer = new byte[BufferSize];

			int HowMuchLeft = UnpackedSize;			
			while(HowMuchLeft > 0) {
				int NumToRead = HowMuchLeft > BufferSize ? BufferSize : HowMuchLeft;
				int ActualNumberRead = IS.read(Buffer,0,NumToRead);
				if (ActualNumberRead == -1) throw new Exception("Premature end-of-file reached");
				DOS.write(Buffer,0,ActualNumberRead);
				HowMuchLeft -= ActualNumberRead;
			}
			
			IS.close();
			DOS.finish();	// Need to do this to finish writing before moving on
			OS.close();
		} catch (Exception X) {
			InfoAlert A =
				new InfoAlert(this, "Processing Error for " + FileName, 480,48, X.toString());
			return;
		}
	}
	
	// For the Quit warning alert
	private Frame Parent;

	public MasterDialog() {
		setTitle("TRUnpacker Master Dialog");
		setSize(384,64);
		setResizable(false);
		
		// Hack to get the Quit warning alert to use this frame
		Parent = this;
		
		// Create the menu:
		
		MenuItem UnpackItem = new MenuItem("Unpack...");
		UnpackItem.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {UnpackFile();}}
		);
		
		MenuItem PackItem = new MenuItem("Pack...");
		PackItem.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {PackFile();}}
		);
		
		MenuItem QuitItem = new MenuItem("Quit");
		QuitItem.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				// Just plain bugging out
				dispose();
				System.exit(0);
			}}
		);
		
		Menu FileMenu = new Menu("File");
		FileMenu.add(UnpackItem);
		FileMenu.add(PackItem);
		FileMenu.add(QuitItem);
		
		MenuBar MB = new MenuBar();
		MB.add(FileMenu);
		setMenuBar(MB);
		
		setLayout(new GridBagLayout());
		add(new Label("TRUnpacker, an Unpacker (and also Packer) of Tomb Raider PAK Files"));
				
		validate();
		setVisible(true);
	}
};